{% extends "!base.html" %}
{% block announce %}
  <p>
    Now includes
    <a href="https://squidfunk.github.io/mkdocs-material/changelog/">changes from mkdocs-material theme</a>
    v8.5.6
  </p>
{% endblock %}
