////
/// Copyright (c) 2016-2022 Martin Donath <martin.donath@squidfunk.com>
///
/// Permission is hereby granted, free of charge, to any person obtaining a
/// copy of this software and associated documentation files (the "Software"),
/// to deal in the Software without restriction, including without limitation
/// the rights to use, copy, modify, merge, publish, distribute, sublicense,
/// and/or sell copies of the Software, and to permit persons to whom the
/// Software is furnished to do so, subject to the following conditions:
///
/// The above copyright notice and this permission notice shall be included in
/// all copies or substantial portions of the Software.
///
/// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
/// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
/// FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
/// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
/// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
/// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
/// DEALINGS
////

// ----------------------------------------------------------------------------
// Rules
// ----------------------------------------------------------------------------

// Was this page helpful?
.md-feedback {
  margin: 2em 0 1em;
  text-align: center;

  // Feedback fieldset
  fieldset {
    margin: 0;
    padding: 0;
    border: none;
  }

  // Feedback title
  &__title {
    margin: 1em auto;
    font-weight: 700;
  }

  // Feedback wrapper
  &__inner {
    position: relative;
  }

  // Feedback list
  &__list {
    position: relative;
    display: flex;
    flex-wrap: wrap;
    align-content: baseline;
    justify-content: center;

    // Feedback icon on hover
    &:hover .md-icon:not(:disabled) {
      color: var(--md-default-fg-color--lighter);
    }

    // Adjust height after submission
    :disabled & {
      min-height: px2rem(36px);
    }
  }

  // Feedback icon
  &__icon {
    flex-shrink: 0;
    margin: 0 px2rem(2px);
    color: var(--md-default-fg-color--light);
    cursor: pointer;
    transition: color 125ms;

    // Feedback icon on hover
    &:not(:disabled).md-icon:hover {
      color: var(--md-accent-fg-color);
    }

    // Feedback icon after submit
    &:disabled {
      color: var(--md-default-fg-color--lightest);
      pointer-events: none;
    }
  }

  // Feedback note
  &__note {
    position: relative;
    transform: translateY(px2rem(8px));
    opacity: 0;
    transition:
      transform 400ms cubic-bezier(0.1, 0.7, 0.1, 1),
      opacity   150ms;

    // Feedback note value
    > * {
      max-width: px2rem(320px);
      margin: 0 auto;
    }

    // Show after submission
    :disabled & {
      transform: translateY(0);
      opacity: 1;
    }
  }
}
