{#-
  This file was automatically generated - do not edit
-#}
{%- if logo -%}
  <img src="{{ logo_url }}" alt="logo">
{%- elif theme_logo_svg -%}
  {%- include theme_logo_svg -%}
{%- else %}
  {% set icon = config.theme.icon.logo or "material/library" %}
  {% include ".icons/" ~ icon ~ ".svg" %}
{%- endif -%}
