{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "sr",
  "clipboard.copy": "Копирај у клипборд",
  "clipboard.copied": "Ископирано у клипборд",
  "edit.link.title": "Ажурирај страницу",
  "footer.previous": "Претходно",
  "footer.next": "Следеће",
  "footer.title": "Подножје",
  "header.title": "Заглавље",
  "meta.comments": "Коментари",
  "meta.source": "Извор",
  "nav.title": "Навигација",
  "search.placeholder": "Претрага",
  "search.share": "Дељење",
  "search.reset": "Очисти",
  "search.result.initializer": "Иницијализујем претрагу",
  "search.result.placeholder": "Унесите појам претраге",
  "search.result.none": "Ништа није пронађено",
  "search.result.one": "1 резултат претраге",
  "search.result.other": "# резултата претраге",
  "search.result.more.one": "још 1 на овој страни",
  "search.result.more.other": "још # на овој страни",
  "search.result.term.missing": "Недостаје",
  "search.title": "Претрага",
  "select.language.title": "Изабери језик",
  "select.version.title": "Изабери верзију",
  "skip.link.title": "Иди на текст",
  "source.link.title": "Иди у репозиторијум",
  "source.file.date.updated": "Ажуриран",
  "source.file.date.created": "Креиран",
  "tabs.title": "Табови",
  "toc.title": "Садржај",
  "top.title": "Назад на врх"
}[key] }}{% endmacro %}
