#!/bin/bash

wget https://raw.githubusercontent.com/p12tic/cppreference-doc/master/index-functions-cpp.xml -O index-functions-cpp.xml
wget https://raw.githubusercontent.com/p12tic/cppreference-doc/master/index-functions-c.xml -O index-functions-c.xml
