////
/// Copyright (c) 2016-2022 Martin Donath <martin.donath@squidfunk.com>
///
/// Permission is hereby granted, free of charge, to any person obtaining a
/// copy of this software and associated documentation files (the "Software"),
/// to deal in the Software without restriction, including without limitation
/// the rights to use, copy, modify, merge, publish, distribute, sublicense,
/// and/or sell copies of the Software, and to permit persons to whom the
/// Software is furnished to do so, subject to the following conditions:
///
/// The above copyright notice and this permission notice shall be included in
/// all copies or substantial portions of the Software.
///
/// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
/// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
/// FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
/// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
/// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
/// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
/// DEALINGS
////

// ----------------------------------------------------------------------------
// Rules
// ----------------------------------------------------------------------------

// Scoped in typesetted content to match specificity of regular content
.md-typeset {

  // Headerlink
  .headerlink {
    display: inline-block;
    margin-inline-start: px2rem(10px);
    color: var(--md-default-fg-color--lighter);
    opacity: 0;
    transition:
      color   250ms,
      opacity 125ms;

    // [print]: Hide headerlinks
    @media print {
      display: none;
    }
  }

  // Show headerlinks on parent hover
  :is(:hover, :target) > .headerlink,
  .headerlink:focus {
    opacity: 1;
    transition:
      color   250ms,
      opacity 125ms;
  }

  // Adjust color on parent target or focus/hover
  :target > .headerlink,
  .headerlink:is(:focus, :hover) {
    color: var(--md-accent-fg-color);
  }

  // Adjust scroll margin for all elements with `id` attributes
  :target {
    --md-scroll-margin: #{px2rem(48px + 24px)};
    --md-scroll-offset: #{px2rem(0px)};
    // Scroll margin is finally ready for prime time - before, we used a hack
    // for anchor correction based on pseudo elements but those times are gone.
    scroll-margin-top:
      calc(
        var(--md-scroll-margin) -
        var(--md-scroll-offset)
      );

    // [screen +]: Sticky navigation tabs
    @include break-from-device(screen) {

      // Adjust scroll margin for sticky navigation tabs
      .md-header--lifted ~ .md-container & {
        --md-scroll-margin: #{px2rem(96px + 24px)};
      }
    }
  }

  // Adjust scroll offset for headlines of level 1-3
  :is(h1, h2, h3):target {
    --md-scroll-offset: #{px2rem(4px)};
  }

  // Adjust scroll offset for headlines of level 4
  h4:target {
    --md-scroll-offset: #{px2rem(3px)};
  }
}
