class Bravo:
    """
    Class Bravo docstring.
    """

    def my_method_Bravo(self) -> int:
        """
        This is my_method_Bravo.

        This is paragraph two.
        """
        pass

    @property
    def my_property_Bravo(self) -> str:
        """
        This is my_property_Bravo.

        This is paragraph two.
        """
        pass
