////
/// Copyright (c) 2016-2020 Martin Donath <martin.donath@squidfunk.com>
///
/// Permission is hereby granted, free of charge, to any person obtaining a
/// copy of this software and associated documentation files (the "Software"),
/// to deal in the Software without restriction, including without limitation
/// the rights to use, copy, modify, merge, publish, distribute, sublicense,
/// and/or sell copies of the Software, and to permit persons to whom the
/// Software is furnished to do so, subject to the following conditions:
///
/// The above copyright notice and this permission notice shall be included in
/// all copies or substantial portions of the Software.
///
/// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
/// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
/// FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
/// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
/// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
/// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
/// DEALINGS
////

// ----------------------------------------------------------------------------
// Rules
// ----------------------------------------------------------------------------

// Hero teaser
.md-hero {
  overflow: hidden;
  color: var(--md-primary-bg-color);
  font-size: px2rem(20px);
  background-color: var(--md-primary-fg-color);
  transition: background 250ms;

  // Inner wrapper
  &__inner {
    margin-top: px2rem(20px);
    padding: px2rem(16px) px2rem(16px) px2rem(8px);
    transition:
      transform 400ms cubic-bezier(0.1, 0.7, 0.1, 1),
      opacity   250ms;
    transition-delay: 100ms;

    // [tablet -]: Compensate for missing tabs
    @include break-to-device(tablet) {
      margin-top: px2rem(48px);
      margin-bottom: px2rem(24px);
    }

    // Fade-out tabs background upon scrolling
    [data-md-state="hidden"] & {
      transform: translateY(px2rem(12.5px));
      opacity: 0;
      transition:
        transform   0ms 400ms,
        opacity   100ms   0ms;
      pointer-events: none;
    }

    // Adjust bottom spacing if there are no tabs
    .md-hero--expand & {
      margin-bottom: px2rem(24px);
    }
  }
}
