////
/// Copyright (c) 2016-2022 Martin Donath <martin.donath@squidfunk.com>
///
/// Permission is hereby granted, free of charge, to any person obtaining a
/// copy of this software and associated documentation files (the "Software"),
/// to deal in the Software without restriction, including without limitation
/// the rights to use, copy, modify, merge, publish, distribute, sublicense,
/// and/or sell copies of the Software, and to permit persons to whom the
/// Software is furnished to do so, subject to the following conditions:
///
/// The above copyright notice and this permission notice shall be included in
/// all copies or substantial portions of the Software.
///
/// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
/// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
/// FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
/// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
/// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
/// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
/// DEALINGS
////

// ----------------------------------------------------------------------------
// Keyframes
// ----------------------------------------------------------------------------

// Show consent
@keyframes consent {
  0% {
    transform: translateY(100%);
    opacity: 0;
  }

  100% {
    transform: translateY(0);
    opacity: 1;
  }
}

// Show consent overlay
@keyframes overlay {
  0% {
    opacity: 0;
  }

  100% {
    opacity: 1;
  }
}

// ----------------------------------------------------------------------------
// Rules
// ----------------------------------------------------------------------------

// Consent
.md-consent {

  // Consent overlay
  &__overlay {
    position: fixed;
    top: 0;
    z-index: 5;
    width: 100%;
    height: 100%;
    background-color: hsla(0, 0%, 0%, 0.54);
    opacity: 1;
    backdrop-filter: blur(px2rem(2px));
    animation: overlay 250ms both;
  }

  // Consent wrapper
  &__inner {
    position: fixed;
    bottom: 0;
    z-index: 5;
    width: 100%;
    max-height: 100%;
    padding: 0;
    overflow: auto;
    background-color: var(--md-default-bg-color);
    border: 0;
    border-radius: px2rem(2px);
    box-shadow:
      0 0           px2rem(4px) rgba(0, 0, 0, 0.1),
      0 px2rem(4px) px2rem(8px) rgba(0, 0, 0, 0.2);
    animation: consent 500ms cubic-bezier(0.1, 0.7, 0.1, 1) both;
  }

  // Consent form
  &__form {
    padding: px2rem(16px);
  }

  // Consent settings
  &__settings {
    display: none;
    margin: 1em 0;

    // Show settings
    input:checked + & {
      display: block;
    }
  }

  // Consent controls
  &__controls {
    margin-bottom: px2rem(16px);

    // Consent control button
    .md-typeset & .md-button {
      display: inline;

      // [tablet +]: Align buttons horizontally
      @include break-to-device(mobile) {
        display: block;
        width: 100%;
        margin-top: px2rem(8px);
        text-align: center;
      }
    }
  }

  // Ensure users realize that labels are clickaböe
  label {
    cursor: pointer;
  }
}
