"""
This type stub file was generated by pyright.
"""
# pylint: disable=multiple-statements
from typing import Any
from docutils import nodes
from docutils.parsers.rst import Directive
from sphinx.transforms.post_transforms import SphinxPostTransform

class details(nodes.Element, nodes.General): ...
class summary(nodes.TextElement, nodes.General): ...

def visit_details(self, node: details) -> None: ...
def depart_details(self, node: details) -> None: ...
def visit_summary(self, node: summary) -> None: ...
def depart_summary(self, node: summary) -> None: ...

class DetailsDirective(Directive):
    required_arguments: int
    final_argument_whitespace: bool
    has_content: bool
    option_spec: dict[str, Any]
    def run(self) -> list[nodes.container]: ...

class DetailsTransform(SphinxPostTransform):
    default_priority: int
    builders: tuple[str, Any]
    def run(self, **kwargs: Any) -> None: ...

def setup(app) -> dict[str, Any]: ...
