{#-
  This file was automatically generated - do not edit
-#}
<footer class="md-footer">
  {%- block footerrel %}
  {% if page.previous_page or page.next_page %}
    {% if page.meta and page.meta.hide %}
      {% set hidden = "hidden" if "footer" in page.meta.hide %}
    {% endif %}
    <nav class="md-footer__inner md-grid" aria-label="{{ lang.t('footer.title') }}" {{ hidden }}>
      {% if page.previous_page %}
        {% set direction = lang.t("footer.previous") %}
        <a href="{{ page.previous_page.url | url }}" class="md-footer__link md-footer__link--prev" aria-label="{{ direction }}: {{ page.previous_page.title | e }}" rel="prev">
          <div class="md-footer__button md-icon">
            {% include ".icons/material/arrow-left.svg" %}
          </div>
          <div class="md-footer__title">
            <div class="md-ellipsis">
              <span class="md-footer__direction">
                {{ direction }}
              </span>
              {{ page.previous_page.title }}
            </div>
          </div>
        </a>
      {% endif %}
      {% if page.next_page %}
        {% set direction = lang.t("footer.next") %}
        <a href="{{ page.next_page.url | url }}" class="md-footer__link md-footer__link--next" aria-label="{{ direction }}: {{ page.next_page.title | e }}" rel="next">
          <div class="md-footer__title">
            <div class="md-ellipsis">
              <span class="md-footer__direction">
                {{ direction }}
              </span>
              {{ page.next_page.title }}
            </div>
          </div>
          <div class="md-footer__button md-icon">
            {% include ".icons/material/arrow-right.svg" %}
          </div>
        </a>
      {% endif %}
    </nav>
  {% endif %}
  {% endblock %}
  {%- if False %}{%- block extracopyright %}{%- endblock extracopyright %}{% endif %}
  {% set rendered_extracopyright = self.extracopyright() %}
  <div class="md-footer-meta md-typeset">
    {% if show_copyright or show_sphinx or config.extra.social or (rendered_extracopyright | trim) or (self.extrafooter() | trim) %}
    <div class="md-footer-meta__inner md-grid">
      {% include "partials/copyright.html" %}
      {% if config.extra.social %}
        {% include "partials/social.html" %}
      {% endif %}
      {%- block extrafooter %}{%- endblock %}
    </div>
    {% endif %}
  </div>
</footer>
